﻿using System;
using System.Collections.Generic;
using System.Text;
using Editor.View;
using Editor.Document;
using Editor.Text;

namespace Editor.Commands.Document.Text
{
    public sealed class RemoveChar : ICommand, IDocumentCommand
    {
        private IDocument document;

        public IDocument Document
        {
            get { return document; }
        }

        public RemoveChar( IDocument document )
        {
            this.document = document;
        }
            
        public bool Execute()
        {
            if( document.Selection.MultiIndex ) return false;

            IDocumentRow documentRow = document[document.Selection.StartRow];

            if( !documentRow.Editable ) return false;

            int startIndex = 0;
            int endIndex = 0;
            int index = documentRow.Text.GetVirtualIndex( document.Selection.StartIndex, 
                out startIndex, out endIndex );

            if( document.Selection.StartIndex != startIndex || index <= 0 ) return false;

            int charLength = documentRow.Text[index - 1].CharacterLength;

            IAppendableCommand command = document.LastCommand as IAppendableCommand;

            if( command != null && charLength == 1 && command.CanAppend( documentRow, index ) )
            {
                IText removed = documentRow.Text.Remove( index - 1, 1 );
                command.AppendText( removed );
            }
            else
            {
                document.Selection.Change( document.Selection.StartRow, document.Selection.StartIndex,
                    document.Selection.StartRow, document.Selection.StartIndex - charLength);

                return document.ExecuteCommand( new RemoveText( document ) );
            }      

            document.Selection.Change( document.Selection.StartRow, document.Selection.StartIndex - charLength,
                document.Selection.StartRow, document.Selection.StartIndex - charLength );

            return true;
        }
    }
}
