﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Editor.Document
{
    public struct SSelection
    {
        private int endIndex;
        private int endRow;
        private int startIndex;
        private int startRow;

        public int EndIndex
        {
            get { return endIndex; }
            set { endIndex = value; }
        }

        public int EndRow
        {
            get { return endRow; }
            set { endRow = value; }
        }

        public bool MultiIndex 
        {
            get 
            {
                if( startRow != endRow ) return true;
                if( startIndex == endIndex ) return false;

                return true;
            }
        }

        public bool MultiRow 
        {
            get 
            {
                if( startRow != endRow ) return true;

                return false;
            }
        }

        public int StartIndex
        {
            get { return startIndex; }
            set { startIndex = value; }
        }

        public int StartRow
        {
            get { return startRow; }
            set { startRow = value; }
        }

        public SSelection( int startRow, int startIndex, int endRow, int endIndex )
        {
            this.startRow = startRow;
            this.endRow = endRow;
            this.startIndex = startIndex;
            this.endIndex = endIndex;
        }

        public void Copy( SSelection source )
        {
            startRow = source.StartRow;
            endRow = source.EndRow;
            startIndex = source.StartIndex;
            endIndex = source.EndIndex;
        }

        public void Change( int startRow, int startIndex, int endRow, int endIndex )
        {
            this.startRow = startRow;
            this.endRow = endRow;
            this.startIndex = startIndex;
            this.endIndex = endIndex;
        }

        public SSelection GetOrdered()
        {
            if( startRow > endRow ) return new SSelection( endRow, endIndex, startRow, startIndex );
            else if( startRow < endRow ) return new SSelection( startRow, startIndex, endRow, endIndex );
            else
            {
                if( startIndex < endIndex ) return new SSelection( startRow, startIndex, endRow, endIndex );
                else return new SSelection( endRow, endIndex, startRow, startIndex );
            }
        }
    }
}
