﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Editor;

namespace Editor.Project
{
    public sealed class Variation : INotifyPropertyChanged, ICloneable<Variation>
    {
        private string normal;
        private string lowercase;
        private string uppercase;

        public bool Defined
        {
            get 
            { 
                if( normal == string.Empty ) return false;

                return true;
            }
        }

        public string Normal
        {
            get 
            { 
                if( normal == string.Empty ) return "Not defined";

                return normal; 
            }
            set
            {
                normal = value.Trim();

                if( normal.Length > 1 )
                {
                    lowercase = string.Concat( char.ToLower( normal[0] ).ToString(), normal.Substring( 1 ) );
                    uppercase = string.Concat( char.ToUpper( normal[0] ).ToString(), normal.Substring( 1 ) );
                }
                else
                {
                    lowercase = normal.ToLower();
                    uppercase = normal.ToUpper();
                }

                if( PropertyChanged != null )
                    PropertyChanged( this, new PropertyChangedEventArgs("Normal") );
            }
        }

        public string Lowercase
        {
            get 
            { 
                if( normal == string.Empty ) return "Not defined";

                return lowercase; 
            }
        }

        public string Uppercase
        {
            get 
            { 
                if( normal == string.Empty ) return "Not defined";

                return uppercase;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public Variation( string normal )
        {
            this.normal = normal.Trim();
            
            if( this.normal.Length > 1 )
            {
                lowercase = string.Concat( char.ToLower( this.normal[0] ).ToString(), this.normal.Substring( 1 ) );
                uppercase = string.Concat( char.ToUpper( this.normal[0] ).ToString(), this.normal.Substring( 1 ) );
            }
            else
            {
                lowercase = this.normal.ToLower();
                uppercase = this.normal.ToUpper();
            }
        }

        private Variation( string normal, string lowercase, string uppercase )
        {
            this.normal = normal;
            this.lowercase = lowercase;
            this.uppercase = uppercase;
        }

        public Variation Clone()
        {
            return new Variation( normal, lowercase, uppercase );
        }

        public override string ToString()
        {
            if( normal == string.Empty ) return "Not defined";

            return normal;
        }
    }
}
