﻿using System;
using System.Collections.Generic;
using System.Text;
using Editor.Project;

namespace Editor.Text.UseCase
{
    public sealed class DefinitionGlyph : Glyph, IVariableGlyph
    {
        private string currentVariation;
        private DictionaryWord dictionaryWord;
        private int variationIndex;
        private EVariationGroup variationGroup;

        public bool Changing
        {
            get { throw new NotImplementedException(); }
        }

        public override int CharacterLength 
        {
            get { return currentVariation.Length; }
        }

        public override bool StyleChanging
        {
            get { return false; }
        }

        public override bool StyleCopying
        {
            get { return false; }
        }

        public override EGlyphType Type
        {
            get { return EGlyphType.Definition; }
        }

        public EVariationGroup VariationGroup
        {
            get { return variationGroup; }
            set 
            { 
                ChangeVariation( variationIndex, value );
            }
        }

        public int VariationIndex
        {
            get { return variationIndex; }
            set 
            { 
                ChangeVariation( value, variationGroup );
            }
        }

        public event EventHandler ObjectChanged;
        public event EventHandler ObjectChanging;

        public DefinitionGlyph( DictionaryWord dictionaryWord, int variationIndex, 
            EVariationGroup variationGroup ) 
        {
            currentVariation = string.Empty;
            this.dictionaryWord = dictionaryWord;
            style = (short)EDefaultStyles.Definition;
            this.variationGroup = variationGroup;
            this.variationIndex = variationIndex;

            ChangeVariation( variationIndex, variationGroup );
        }

        public void ChangeVariation( int index, EVariationGroup group )
        {
            variationGroup = group;
            variationIndex = index;

            string newVariation = string.Empty;

            if( variationIndex < 0 )
                newVariation = dictionaryWord.Name;
            else
            {
                switch( group )
                {
                    case EVariationGroup.Lowercase: newVariation = dictionaryWord.Variations[index].Lowercase;
                        break;
                    case EVariationGroup.Normal: newVariation = dictionaryWord.Variations[index].Normal;
                        break;
                    case EVariationGroup.Uppercase: newVariation = dictionaryWord.Variations[index].Uppercase;
                        break;
                    default: newVariation = dictionaryWord.Name;
                        break;
                }
            }

            if( ObjectChanged != null )
            {
                if( !string.Equals( currentVariation, newVariation, StringComparison.CurrentCulture ) )
                {
                    currentVariation = newVariation;
                    ObjectChanged( dictionaryWord, EventArgs.Empty );
                }
            }
            else currentVariation = newVariation;
        }

        public void BeginChange()
        {
            throw new NotImplementedException();
        }

        public override Glyph Clone()
        {
            return new DefinitionGlyph( dictionaryWord, variationIndex, variationGroup );
        }

        public void EndChange( bool discardEvent )
        {
            throw new NotImplementedException();
        }
        
        public Delegate[] GetReferences()
        {
            return ObjectChanged.GetInvocationList();
        }

        public override char GetCharAt( int index )
        {
            return currentVariation[index];
        }

        public override string ToString()
        {
            return currentVariation;
        }
    }
}
