﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Editor.Document;

namespace Editor.View
{
    public sealed class Caret : ICaret
    {
        private Point caretEnd;
        private Pen caretPen;
        private Point caretStart;
        private bool caretVisible;
        private Timer caretTimer;
        private IView view;

        public Color Color
        {
            get { return caretPen.Color; }
            set { caretPen.Color = value; }
        }

        public int Interval 
        {
            get { return caretTimer.Interval; }
            set { caretTimer.Interval = value; }
        }

        public Caret( IView view, int interval, Color color )
        {
            this.caretEnd = Point.Empty;
            this.caretPen = new Pen( color );
            this.caretStart = Point.Empty;
            this.caretVisible = false;
            this.caretTimer = new Timer();
            this.caretTimer.Interval = interval;
            this.caretTimer.Tick += new EventHandler( CaretTimerTick );
            this.view = view;
        }

        public void Draw( Arguments arguments )
        {
            PaintArguments paintArguments = arguments as PaintArguments;

            if( caretVisible )
            {
                paintArguments.Graphics.DrawLine( caretPen, caretStart, caretEnd );
            }
        }

        public void Hide()
        {
            if( caretTimer.Enabled ) caretTimer.Stop();
            caretVisible = false;
        }

        public void Place( int x, int y, int height, bool visible )
        {
            if( caretTimer.Enabled ) caretTimer.Stop(); 

            caretStart.X = x;
            caretStart.Y = y;
            caretEnd.X = x;
            caretEnd.Y = y + height;
            caretVisible = visible;

            caretTimer.Start();
        }

        public void Show()
        {
            if( caretTimer.Enabled ) caretTimer.Stop();

            caretVisible = true;
            caretTimer.Start();
        }

        private void CaretTimerTick( object sender, EventArgs e )
        {
            if( caretVisible == true ) caretVisible = false;
            else caretVisible = true;

            view.Redraw();
        }
    }
}
