﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Editor.Document;

namespace UseCaseTool.Controls
{
    public class RowsListBox : ListBox
    {
        protected IDocument document;

        public IDocument Document 
        {
            get { return document; }
            set 
            {
                if( document != null ) document.ObjectChanged -= OnDocumentChanged;

                document = value;

                if( document == null ) Items.Clear();
                else
                {
                    document.ObjectChanged += OnDocumentChanged;
                    Repopulate();
                }
            }
        }

        public RowsListBox( IDocument document )
        {
            this.document = document;
            if( document != null ) 
            {
                document.ObjectChanged += OnDocumentChanged;
                Repopulate();
            }
        }

        protected virtual void OnDocumentChanged( object sender, EventArgs eventArgs )
        {
            Repopulate();
        }

        public void Repopulate()
        {
            if( document == null ) return;

            BeginUpdate();

            Items.Clear();

            for( int i = 0; i < document.RowCount; i++ )
            {
                IDocumentRow documentRow = document[i];

                if( ApplyFilter( documentRow ) )Items.Add( new ListBoxDocumentRow( documentRow ) );
            }

            EndUpdate();
        }

        protected virtual bool ApplyFilter( IDocumentRow documentRow )
        {
            return true;
        }
    }
}
