﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Editor.Text;
using Editor.Project;
using UseCaseTool.Controls;

namespace UseCaseTool.Forms
{
    public partial class ProjectSettingsForm : Form
    {
        private UseCaseProject project;

        public ProjectSettingsForm()
        {
            InitializeComponent();

            project = null;
        }

        public ProjectSettingsForm( UseCaseProject project )
        {
            InitializeComponent();

            this.project = project;

            projectNameTB.Text = project.ItemName;
            projectDescriptionTB.Text = project.Description;

            InitializeConfiguration();
            InitializeDefaltColors();
            InitializeCustomColors();
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            if( projectNameTB.Text.Trim() != string.Empty ) 
                project.ItemName = projectNameTB.Text.Trim();

            project.Description = projectDescriptionTB.Text.Trim();

            base.OnClosing( e );
        }

        private void InitializeConfiguration()
        {
            option1CB.Checked = project.ModelProperties[0];
            option2CB.Checked = project.ModelProperties[1];
            option3CB.Checked = project.ModelProperties[2];
            option4CB.Checked = project.ModelProperties[3];
            option5CB.Checked = project.ModelProperties[4];
            option6CB.Checked = project.ModelProperties[5];
            option7CB.Checked = project.ModelProperties[6];
            option8CB.Checked = project.ModelProperties[7];
            option9CB.Checked = project.ModelProperties[8];
            option10CB.Checked = project.ModelProperties[9];
            option11CB.Checked = project.ModelProperties[10];
            option12CB.Checked = project.ModelProperties[11];
            option13CB.Checked = project.ModelProperties[12];
            option14CB.Checked = project.ModelProperties[13];
            option15CB.Checked = project.ModelProperties[14];
            option16CB.Checked = project.ModelProperties[15];
            option17CB.Checked = project.ModelProperties[16];
            option18CB.Checked = project.ModelProperties[17];
            option19CB.Checked = project.ModelProperties[18];
            option20CB.Checked = project.ModelProperties[19];

            if( project.ItemCount > 0 )
            {
                option1CB.Enabled = false;
                option2CB.Enabled = false;
                option3CB.Enabled = false;
                option4CB.Enabled = false;
                option5CB.Enabled = false;
                option6CB.Enabled = false;
                option7CB.Enabled = false;
                option8CB.Enabled = false;
                option9CB.Enabled = false;
                option10CB.Enabled = false;
                option11CB.Enabled = false;
                option12CB.Enabled = false;
                option13CB.Enabled = false;
                option14CB.Enabled = false;
                option15CB.Enabled = false;
                option16CB.Enabled = false;
                option17CB.Enabled = false;
                option18CB.Enabled = false;
                option19CB.Enabled = false;
                option20CB.Enabled = false;

                addDescrItemBN.Enabled = false;
                addStepBN.Enabled = false;
                changeDescrItemBN.Enabled = false;
                changeStepBN.Enabled = false;
                removeDescrItemBN.Enabled = false;
                removeStepBN.Enabled = false;

                specialStepTB.ReadOnly = true;
                descriptionItemTB.ReadOnly = true;
            }

            descriptionItemsLB.DataSource = project.DescriptionItems;
            specialStepsLB.DataSource = project.SpecialSteps;     
        }

        private void InitializeCustomColors() 
        {
            for( short i = project.StyleTable.DefaultStyles; i <= project.StyleTable.MaximumIndex; i++ )
            {
                if( project.StyleTable.Contains( i ) )
                {
                    StyleButton button = new StyleButton( project.StyleTable[i], i );
                    button.FlatStyle = FlatStyle.Flat;
                    button.Size = new Size( 300, 30 );
                    button.Click += new EventHandler( button_Click );
                    button.MouseDown += new MouseEventHandler( button_MouseDown );

                    customColorsFLP.Controls.Add( button );
                }
            }
        }

        private void InitializeDefaltColors()
        {
            for( short i = 0; i < project.StyleTable.DefaultStyles; i++ )
            {
                StyleButton button = new StyleButton( project.StyleTable[i], i );
                button.FlatStyle = FlatStyle.Flat;
                button.Size = new Size( 300, 30 );
                button.Click += new EventHandler( button_Click );

                defaultColorsFLP.Controls.Add( button );
            }
        }

        private void button_Click(object sender, EventArgs e)
        {
            StyleButton button = sender as StyleButton;

            DialogResult dialogResult = new StylesEditorForm( button.Style ).ShowDialog();

            if( dialogResult == DialogResult.OK ) 
            {
                button.UpdateStyle();

                project.Saved = false;
            }
        }


        private void button_MouseDown(object sender, MouseEventArgs e)
        {
            if( e.Button != MouseButtons.Right ) return;

            StyleButton button = sender as StyleButton;

            DialogResult dialogResult = MessageBox.Show( "Delete style?", "Question", 
                MessageBoxButtons.YesNo, MessageBoxIcon.Question );

            if( dialogResult == DialogResult.Yes ) 
            {
                project.StyleTable.Remove( button.Index );
                customColorsFLP.Controls.Remove( button );

                project.Saved = false;
            }
        }


        private void addStepBN_Click(object sender, EventArgs e)
        {
            if( specialStepTB.Text.Trim() == string.Empty ) return;

            project.SpecialSteps.Add( new SpecialStep( specialStepTB.Text.Trim() ) );

            specialStepTB.Text = string.Empty;
        }

        private void changeStepBN_Click(object sender, EventArgs e)
        {
            if( specialStepTB.Text.Trim() == string.Empty ) return;
            if( specialStepsLB.SelectedIndex < 0 ) return;

            SpecialStep specialStep = (SpecialStep)specialStepsLB.SelectedItem;

            if( specialStep != null )
            {
                specialStep.Name = specialStepTB.Text.Trim();

                specialStepTB.Text = string.Empty;
            }
        }

        private void removeStepBN_Click(object sender, EventArgs e)
        {
            if( specialStepsLB.SelectedIndex < 0 ) return;

            project.SpecialSteps.RemoveAt( specialStepsLB.SelectedIndex );
        }

        private void specialStepsLB_SelectedIndexChanged(object sender, EventArgs e)
        {
            if( specialStepsLB.SelectedIndex < 0 ) return;

            SpecialStep specialStep = (SpecialStep)specialStepsLB.SelectedItem;

            if( specialStep != null )
                specialStepTB.Text = specialStep.Name;
        }

        private void addDescrItemBN_Click(object sender, EventArgs e)
        {
            if( descriptionItemTB.Text.Trim() == string.Empty ) return;

            project.DescriptionItems.Add( new DescriptionItem( descriptionItemTB.Text.Trim() ) );

            descriptionItemTB.Text = string.Empty;
        }

        private void changeDescrItemBN_Click(object sender, EventArgs e)
        {
            if( descriptionItemTB.Text.Trim() == string.Empty ) return;
            if( specialStepsLB.SelectedIndex < 0 ) return;

            DescriptionItem descriptionItem = (DescriptionItem)descriptionItemsLB.SelectedItem;

            if( descriptionItem != null )
            {
                descriptionItem.Name = descriptionItemTB.Text.Trim();

                descriptionItemTB.Text = string.Empty;
            }
        }

        private void removeDescrItemBN_Click(object sender, EventArgs e)
        {
            if( descriptionItemsLB.SelectedIndex < 0 ) return;

            project.DescriptionItems.RemoveAt( descriptionItemsLB.SelectedIndex );
        }

        private void descriptionItemsLB_SelectedIndexChanged(object sender, EventArgs e)
        {
            if( specialStepsLB.SelectedIndex < 0 ) return;

            DescriptionItem descriptionItem = (DescriptionItem)descriptionItemsLB.SelectedItem;

            if( descriptionItem != null )
                descriptionItemTB.Text = descriptionItem.Name;
        }

        private void addColorBN_Click(object sender, EventArgs e)
        {
            for( short i = project.StyleTable.DefaultStyles; i <= project.StyleTable.MaximumIndex; i++ )
            {
                if( !project.StyleTable.Contains( i ) )
                {
                    Style style = new Style();

                    StyleButton button = new StyleButton( style, i );
                    button.FlatStyle = FlatStyle.Flat;
                    button.Size = new Size( 300, 30 );
                    button.Click += new EventHandler( button_Click );
                    button.MouseDown += new MouseEventHandler( button_MouseDown );

                    customColorsFLP.Controls.Add( button );

                    project.StyleTable.Add( i, style );
                    project.Saved = false;

                    return;
                }
            }
        }

        private void option1CB_CheckedChanged( object sender, EventArgs e )
        {
            project.ModelProperties[0] = option1CB.Checked;
            project.Saved = false;
        }

        private void option2CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[1] = option2CB.Checked;
            project.Saved = false;
        }

        private void option3CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[2] = option3CB.Checked;
            project.Saved = false;
        }

        private void option4CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[3] = option4CB.Checked;
            project.Saved = false;
        }

        private void option5CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[4] = option5CB.Checked;
            project.Saved = false;
        }

        private void option6CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[5] = option6CB.Checked;
            project.Saved = false;
        }

        private void option7CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[6] = option7CB.Checked;
            project.Saved = false;
        }

        private void option8CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[7] = option8CB.Checked;
            project.Saved = false;
        }

        private void option9CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[8] = option9CB.Checked;
            project.Saved = false;
        }

        private void option10CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[9] = option10CB.Checked;
            project.Saved = false;
        }

        private void option11CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[10] = option11CB.Checked;
            project.Saved = false;
        }

        private void option12CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[11] = option12CB.Checked;
            project.Saved = false;
        }

        private void option13CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[12] = option13CB.Checked;
            project.Saved = false;
        }

        private void option14CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[13] = option14CB.Checked;
            project.Saved = false;
        }

        private void option15CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[14] = option15CB.Checked;
            project.Saved = false;
        }

        private void option16CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[15] = option16CB.Checked;
            project.Saved = false;
        }

        private void option17CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[16] = option17CB.Checked;
            project.Saved = false;
        }

        private void option18CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[17] = option18CB.Checked;
            project.Saved = false;
        }

        private void option19CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[18] = option19CB.Checked;
            project.Saved = false;
        }

        private void option20CB_CheckedChanged(object sender, EventArgs e)
        {
            project.ModelProperties[19] = option20CB.Checked;
            project.Saved = false;
        }
    }
}
